@@ -1,13 +1,11 @@
-"""
-Esmerald: Highly scalable, performant, easy to learn and for every application.
-"""
-__version__ = "0.11.0"
+__version__ = "2.3.0"
+
 
 from starlette import status
 
 from esmerald.conf import settings
 from esmerald.conf.global_settings import EsmeraldAPISettings
-from esmerald.injector import Inject
+from esmerald.injector import Factory, Inject
 
 from .applications import ChildEsmerald, Esmerald
 from .backgound import BackgroundTask, BackgroundTasks
@@ -27,14 +25,26 @@ from .interceptors.interceptor import EsmeraldInterceptor
 from .param_functions import DirectInjects
 from .params import Body, Cookie, File, Form, Header, Injects, Param, Path, Query
 from .permissions import AllowAny, BasePermission, DenyAll
+from .pluggables import Extension, Pluggable
 from .protocols import AsyncDAOProtocol, DaoProtocol, MiddlewareProtocol
 from .requests import Request
 from .responses import JSONResponse, Response, TemplateResponse
-from .routing.gateways import Gateway, WebSocketGateway
-from .routing.handlers import delete, get, patch, post, put, route, websocket
+from .routing.apis import APIView, SimpleAPIView
+from .routing.gateways import Gateway, WebhookGateway, WebSocketGateway
+from .routing.handlers import delete, get, head, options, patch, post, put, route, trace, websocket
 from .routing.router import Include, Router
-from .routing.views import APIView
-from .websockets import WebSocket, WebSocketDisconnect
+from .routing.webhooks import (
+    whdelete,
+    whead,
+    whget,
+    whoptions,
+    whpatch,
+    whpost,
+    whput,
+    whroute,
+    whtrace,
+)
+from .websockets import WebSocket, WebSocketDisconnect, WebSocketState
 
 __all__ = [
     "AllowAny",
@@ -54,6 +64,7 @@ __all__ = [
     "Esmerald",
     "EsmeraldAPISettings",
     "EsmeraldInterceptor",
+    "Extension",
     "File",
     "Form",
     "Gateway",
@@ -61,6 +72,7 @@ __all__ = [
     "HTTPException",
     "Include",
     "Inject",
+    "Factory",
     "Injects",
     "ImproperlyConfigured",
     "JSON",
@@ -73,6 +85,7 @@ __all__ = [
     "Param",
     "Path",
     "PermissionDenied",
+    "Pluggable",
     "Query",
     "Redirect",
     "Request",
@@ -80,22 +93,37 @@ __all__ = [
     "Router",
     "ServiceUnavailable",
     "SessionConfig",
+    "SimpleAPIView",
     "StaticFilesConfig",
     "Stream",
     "Template",
     "TemplateResponse",
     "UploadFile",
     "ValidationErrorException",
+    "WebhookGateway",
     "WebSocket",
     "WebSocketDisconnect",
     "WebSocketGateway",
+    "WebSocketState",
     "delete",
     "get",
+    "head",
+    "options",
     "patch",
     "post",
     "put",
     "route",
     "settings",
     "status",
+    "trace",
     "websocket",
+    "whdelete",
+    "whead",
+    "whget",
+    "whoptions",
+    "whpatch",
+    "whpost",
+    "whput",
+    "whroute",
+    "whtrace",
 ]
