@@ -4,7 +4,8 @@ from typing import Any, Dict, Optional, Type, Union
 from pydantic import BaseModel, create_model
 from starlette import status
 from starlette.exceptions import HTTPException as StarletteHTTPException
-from starlette.exceptions import WebSocketException
+from starlette.exceptions import WebSocketException as StarletteWebSocketException
+from typing_extensions import Annotated, Doc
 
 RequestErrorModel: Type[BaseModel] = create_model("Request")
 WebSocketErrorModel: Type[BaseModel] = create_model("WebSocket")
@@ -15,7 +16,7 @@ class EsmeraldAPIException(Exception):
         self.detail = detail
         super().__init__(*(str(arg) for arg in args if arg), self.detail)
 
-    def __repr__(self) -> str:
+    def __repr__(self) -> str:  # pragma: no cover
         if self.detail:
             return f"{self.__class__.__name__} - {self.detail}"
         return self.__class__.__name__
@@ -26,20 +27,58 @@ class EsmeraldAPIException(Exception):
 
 class HTTPException(StarletteHTTPException, EsmeraldAPIException):
     """
-    Implementation of a unique exception allowing to override
-    the details.
+    Base of all `Esmerald` execeptions.
+
+    !!! Tip
+        For an implementation of a custom exception that you need
+        to be thrown by Esmerald, it is advised to subclass `HTTPException`.
     """
 
     status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
 
     def __init__(
         self,
-        *args: Any,
-        detail: Optional[str] = None,
-        status_code: Optional[int] = None,
-        headers: Optional[Dict[str, Any]] = None,
-        **extra,
-    ):
+        *args: Annotated[
+            Any,
+            Doc(
+                """
+                The args passed to the exception.
+                """
+            ),
+        ],
+        detail: Annotated[
+            Optional[str],
+            Doc(
+                """
+                A string text with the details of the error being thrown.
+                """
+            ),
+        ] = None,
+        status_code: Annotated[
+            Optional[int],
+            Doc(
+                """
+                An integer with the status code to be raised.
+                """
+            ),
+        ] = None,
+        headers: Annotated[
+            Optional[Dict[str, Any]],
+            Doc(
+                """
+                Any python dictionary containing headers.
+                """
+            ),
+        ] = None,
+        **extra: Annotated[
+            Any,
+            Doc(
+                """
+                Any extra information used by the exception.
+                """
+            ),
+        ],
+    ) -> None:
         detail = detail or getattr(self, "detail", None)
         status_code = status_code or getattr(self, "status_code", None)
         if not detail:
@@ -51,7 +90,7 @@ class HTTPException(StarletteHTTPException, EsmeraldAPIException):
         self.args = (f"{self.status_code}: {self.detail}", *args)
         self.extra = extra
 
-    def __repr__(self) -> str:
+    def __repr__(self) -> str:  # pragma: no cover
         return f"<{self.status_code}: {self.__class__.__name__} />"
 
 
@@ -117,11 +156,19 @@ class MissingDependency(EsmeraldAPIException, ImportError):
     ...
 
 
-class OpenAPIError(ValueError):
+class OpenAPIException(ImproperlyConfigured):
     ...
 
 
-class WebSocketException(WebSocketException):
+class WebSocketException(StarletteWebSocketException):
+    ...
+
+
+class AuthenticationError(HTTPException):
+    status_code = status.HTTP_401_UNAUTHORIZED
+
+
+class EnvironmentError(EsmeraldAPIException):
     ...
 
 
