@@ -1,12 +1,7 @@
-from typing import TYPE_CHECKING, Any, List, TypeVar
+from typing import Any, List
 
 from typing_extensions import Protocol, runtime_checkable
 
-if TYPE_CHECKING:
-    from esmerald.types import DictAny
-
-T = TypeVar("T")
-
 
 @runtime_checkable
 class DaoProtocol(Protocol):  # pragma: no cover
@@ -17,26 +12,24 @@ class DaoProtocol(Protocol):  # pragma: no cover
     Esmerald definition of a DAO will contain the base patterns for an object
     """
 
-    @property
-    def model(self) -> Any:
-        ...
+    model: Any
 
     """
     The model object. Usually this object is what allows the connection and
     data access to a database.
     """
 
-    def get(self, obj_id: Any, **kwargs: "DictAny") -> Any:
+    def get(self, obj_id: Any, **kwargs: Any) -> Any:
         ...
 
-    def get_all(self, **kwargs: "DictAny") -> List[Any]:
+    def get_all(self, **kwargs: Any) -> List[Any]:
         ...
 
-    def update(self, obj_id: Any, **kwargs: "DictAny") -> Any:
+    def update(self, obj_id: Any, **kwargs: Any) -> Any:
         ...
 
-    def delete(self, obj_id: Any, **kwargs: "DictAny") -> Any:
+    def delete(self, obj_id: Any, **kwargs: Any) -> Any:
         ...
 
-    def create(self, **kwargs: "DictAny") -> Any:
+    def create(self, **kwargs: Any) -> Any:
         ...
