@@ -1,13 +1,10 @@
+import dataclasses
+from dataclasses import is_dataclass
 from typing import Any, Dict
 
-from orjson import OPT_OMIT_MICROSECONDS  # noqa
 from pydantic import BaseModel
-from starlette.responses import JSONResponse as JSONResponse  # noqa
 
-try:
-    import orjson
-except ImportError:  # pragma: nocover
-    orjson = None  # type: ignore
+from esmerald.responses import JSONResponse as JSONResponse  # noqa
 
 
 class BaseJSONResponse(JSONResponse):
@@ -16,11 +13,13 @@ class BaseJSONResponse(JSONResponse):
     """
 
     @staticmethod
-    def transform(value: Any) -> Dict[str, Any]:
+    def transform(value: Any) -> Dict[str, Any]:  # pragma: no cover
         """
         Makes sure that every value is checked and if it's a pydantic model then parses into
         a dict().
         """
         if isinstance(value, BaseModel):
-            return value.dict()
+            return value.model_dump()
+        if is_dataclass(value):
+            return dataclasses.asdict(value)
         raise TypeError("unsupported type")
