@@ -13,7 +13,6 @@ from typing import (
     List,
     Optional,
     Set,
-    Tuple,
     Type,
     TypeVar,
     Union,
@@ -23,7 +22,6 @@ from uuid import UUID
 
 from starlette.convertors import CONVERTOR_TYPES
 from starlette.requests import HTTPConnection
-from starlette.responses import JSONResponse
 from starlette.responses import Response as StarletteResponse
 from starlette.routing import Mount as Mount  # noqa
 from starlette.routing import compile_path
@@ -38,29 +36,31 @@ from esmerald.injector import Inject
 from esmerald.permissions.utils import continue_or_raise_permission_exception
 from esmerald.requests import Request
 from esmerald.responses import JSONResponse, Response
-from esmerald.routing.views import APIView
+from esmerald.routing.apis.base import View
 from esmerald.transformers.model import TransformerModel
 from esmerald.transformers.signature import SignatureFactory
 from esmerald.transformers.utils import get_signature
-from esmerald.typing import Void
+from esmerald.typing import Void, VoidType
 from esmerald.utils.helpers import is_async_callable, is_class_and_subclass
 from esmerald.utils.sync import AsyncCallable
 
-if TYPE_CHECKING:
-    from pydantic.typing import AnyCallable
+if TYPE_CHECKING:  # pragma: no cover
+    from openapi_schemas_pydantic.v3_1_0.security_scheme import SecurityScheme
 
     from esmerald.applications import Esmerald
+    from esmerald.interceptors.interceptor import EsmeraldInterceptor
     from esmerald.interceptors.types import Interceptor
+    from esmerald.permissions import BasePermission
     from esmerald.permissions.types import Permission
-    from esmerald.routing.router import HTTPHandler, WebSocketHandler
+    from esmerald.routing.router import HTTPHandler
     from esmerald.types import (
+        APIGateHandler,
         AsyncAnyCallable,
-        ExceptionHandlers,
-        ParentType,
+        Dependencies,
         ResponseCookies,
         ResponseHeaders,
-        ResponseType,
     )
+    from esmerald.typing import AnyCallable
 
 param_type_map = {
     "str": str,
@@ -87,8 +87,8 @@ class PathParameterSchema(TypedDict):
     type: Type
 
 
-class OpenAPIDefinitionMixin:
-    def parse_path(self, path: str) -> Tuple[str, str, List[Union[str, PathParameterSchema]]]:
+class OpenAPIDefinitionMixin:  # pragma: no cover
+    def parse_path(self, path: str) -> List[Union[str, PathParameterSchema]]:
         """
         Using the Starlette CONVERTORS and the application registered convertors,
         transforms the path into a PathParameterSchema used for the OpenAPI definition.
@@ -163,11 +163,11 @@ class BaseResponseHandler:
         """Creates a handler for ResponseContainer Types"""
 
         async def response_content(
-            data: ResponseContainer, app: "Esmerald", **kwargs: Dict[str, Any]
+            data: ResponseContainer, app: Type["Esmerald"], **kwargs: Dict[str, Any]
         ) -> StarletteResponse:
             _headers = {**self.get_headers(headers), **data.headers}
             _cookies = self.get_cookies(data.cookies, cookies)
-            response = data.to_response(
+            response: Response = data.to_response(
                 app=app,
                 headers=_headers,
                 status_code=status_code,
@@ -225,7 +225,7 @@ class BaseResponseHandler:
                 **self.allow_header,
             }
             for cookie in _cookies:
-                data.set_cookie(**cookie)
+                data.set_cookie(**cookie)  # pragma: no cover
 
             for header, value in _headers.items():
                 data.headers[header] = value
@@ -246,7 +246,6 @@ class BaseResponseHandler:
         async def response_content(
             data: StarletteResponse, **kwargs: Dict[str, Any]
         ) -> StarletteResponse:
-
             _cookies = self.get_cookies(cookies, [])
             _headers = {
                 **self.get_headers(headers),
@@ -254,25 +253,24 @@ class BaseResponseHandler:
                 **self.allow_header,
             }
             for cookie in _cookies:
-                data.set_cookie(**cookie)
+                data.set_cookie(**cookie)  # pragma: no cover
 
             for header, value in _headers.items():
                 data.headers[header] = value
-            return data  # type: ignore
+            return data
 
         return response_content
 
-    def handler(
+    def _handler(
         self,
         background: Optional[Union["BackgroundTask", "BackgroundTasks"]],
         cookies: "ResponseCookies",
         headers: Dict[str, Any],
         media_type: str,
-        response_class: "ResponseType",
+        response_class: Any,
         status_code: int,
     ) -> "AsyncAnyCallable":
         async def response_content(data: Any, **kwargs: Dict[str, Any]) -> StarletteResponse:
-
             data = await self.get_response_data(data=data)
             _cookies = self.get_cookies(cookies, [])
             if isinstance(data, JSONResponse):
@@ -289,8 +287,8 @@ class BaseResponseHandler:
                 )
 
             for cookie in _cookies:
-                response.set_cookie(**cookie)
-            return response  # type: ignore
+                response.set_cookie(**cookie)  # pragma: no cover
+            return response
 
         return response_content
 
@@ -298,7 +296,7 @@ class BaseResponseHandler:
         self,
         scope: "Scope",
         request: Request,
-        route: Union["HTTPHandler", "WebSocketHandler"],
+        route: "HTTPHandler",
         parameter_model: "TransformerModel",
     ) -> "StarletteResponse":
         response: Optional["StarletteResponse"] = None
@@ -316,17 +314,14 @@ class BaseResponseHandler:
         self,
         scope: "Scope",
         request: Request,
-        route: Union["HTTPHandler", "WebSocketHandler"],
+        route: "HTTPHandler",
         parameter_model: "TransformerModel",
-    ) -> "StarletteResponse":
-        response_data = None
-
-        if not response_data:
-            response_data = await self._get_response_data(
-                route=route,
-                parameter_model=parameter_model,
-                request=request,
-            )
+    ) -> Any:
+        response_data = await self._get_response_data(
+            route=route,
+            parameter_model=parameter_model,
+            request=request,
+        )
 
         return await self.to_response(
             app=scope["app"],
@@ -359,7 +354,7 @@ class BaseResponseHandler:
             )
         else:
             parsed_kwargs = {}
-        if isinstance(route.parent, APIView):
+        if isinstance(route.parent, View):
             fn = partial(
                 cast("AnyCallable", route.fn),
                 route.parent,
@@ -413,7 +408,7 @@ class BaseResponseHandler:
                     headers=headers,
                 )
             else:
-                handler = self.handler(
+                handler = self._handler(
                     background=self.background,
                     cookies=cookies,
                     headers=headers,
@@ -451,17 +446,21 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
         return parameters
 
     @property
-    def normalised_path_params(self) -> List[Dict[str, str]]:
+    def stringify_parameters(self) -> List[str]:  # pragma: no cover
         """
-        Gets the path parameters in a PathParameterSchema format and it is
-        only used for OpenAPI documentation purposes only.
+        Gets the param:type in string like list.
+        Used for the directive `esmerald show_urls`.
         """
         path_components = self.parse_path(self.path)
         parameters = [component for component in path_components if isinstance(component, dict)]
-        return parameters
+
+        stringified_parameters = [
+            f"{param['name']}:{param['type'].__name__}" for param in parameters
+        ]
+        return stringified_parameters
 
     @property
-    def parent_levels(self) -> List[Union[T, "ParentType"]]:
+    def parent_levels(self) -> List[Any]:
         """
         Returns the handler from the app down to the route handler.
 
@@ -493,21 +492,21 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
         level_dependencies = (level.dependencies or {} for level in self.parent_levels)
         return {name for level in level_dependencies for name in level.keys()}
 
-    def get_permissions(self) -> List["Permission"]:
+    def get_permissions(self) -> List["AsyncCallable"]:
         """
         Returns all the permissions in the handler scope from the ownsership layers.
         """
         if self._permissions is Void:
-            self._permissions = []
+            self._permissions: Union[List["Permission"], "VoidType"] = []
             for layer in self.parent_levels:
                 self._permissions.extend(layer.permissions or [])
             self._permissions = cast(
                 "List[Permission]",
                 [AsyncCallable(permissions) for permissions in self._permissions],
             )
-        return cast("List[Permission]", self._permissions)
+        return cast("List[AsyncCallable]", self._permissions)
 
-    def get_dependencies(self) -> Dict[str, Inject]:
+    def get_dependencies(self) -> "Dependencies":
         """
         Returns all dependencies of the handler function's starting from the parent levels.
         """
@@ -515,8 +514,9 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
             raise RuntimeError(
                 "get_dependencies cannot be called before a signature model has been generated"
             )
-        if self._dependencies is Void:
-            self._dependencies = {}
+
+        if not self._dependencies or self._dependencies is Void:
+            self._dependencies: "Dependencies" = {}
             for level in self.parent_levels:
                 for key, value in (level.dependencies or {}).items():
                     self.is_unique_dependency(
@@ -525,10 +525,10 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
                         injector=value,
                     )
                     self._dependencies[key] = value
-        return cast("Dict[str, Inject]", self._dependencies)
+        return self._dependencies
 
     @staticmethod
-    def is_unique_dependency(dependencies: Dict[str, Inject], key: str, injector: Inject) -> None:
+    def is_unique_dependency(dependencies: "Dependencies", key: str, injector: Inject) -> None:
         """
         Validates that a given inject has not been already defined under a
         different key in any of the levels.
@@ -540,19 +540,9 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
                     f"If you wish to override a inject, it must have the same key."
                 )
 
-    def get_exception_handlers(self) -> "ExceptionHandlers":
-        """
-        Resolves the exception_handlers by starting from the route handler
-        and moving up.
-        """
-        resolved_exception_handlers = {}
-        for level in self.parent_levels:
-            resolved_exception_handlers.update(level.exception_handlers or {})
-        return resolved_exception_handlers
-
     def get_cookies(
         self, local_cookies: "ResponseCookies", other_cookies: "ResponseCookies"
-    ) -> List[Dict[str, Any]]:
+    ) -> List[Dict[str, Any]]:  # pragma: no cover
         """
         Returns a unique list of cookies.
         """
@@ -562,7 +552,9 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
                 filtered_cookies.append(cookie)
         normalized_cookies: List[Dict[str, Any]] = []
         for cookie in filtered_cookies:
-            normalized_cookies.append(cookie.dict(exclude_none=True, exclude={"description"}))
+            normalized_cookies.append(
+                cookie.model_dump(exclude_none=True, exclude={"description"})
+            )
         return normalized_cookies
 
     def get_headers(self, headers: "ResponseHeaders") -> Dict[str, Any]:
@@ -571,7 +563,7 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
         """
         return {k: v.value for k, v in headers.items()}
 
-    async def get_response_data(self, data: Any) -> Any:
+    async def get_response_data(self, data: Any) -> Any:  # pragma: no cover
         """
         Retrives the response data for sync and async.
         """
@@ -579,7 +571,7 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
             data = await data
         return data
 
-    async def allow_connection(self, connection: "HTTPConnection") -> None:
+    async def allow_connection(self, connection: "HTTPConnection") -> None:  # pragma: no cover
         """
         Validates the connection.
 
@@ -588,25 +580,50 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
         Raises a PermissionDenied exception if not allowed..
         """
         for permission in self.get_permissions():
-            permission = await permission()
-            permission.has_permission(connection, self)
-            continue_or_raise_permission_exception(connection, self, permission)
+            awaitable: "BasePermission" = cast("BasePermission", await permission())
+            request: "Request" = cast("Request", connection)
+            handler = cast("APIGateHandler", self)
+            await continue_or_raise_permission_exception(request, handler, awaitable)
+
+    def get_security_schemes(self) -> List["SecurityScheme"]:
+        """
+        Returns all security schemes from every level.
+        """
+        security_schemes: List["SecurityScheme"] = []
+        for layer in self.parent_levels:
+            security_schemes.extend(layer.security or [])
+        return security_schemes
+
+    def get_handler_tags(self) -> List[str]:
+        """
+        Returns all the tags associated with the handler
+        by checking the parents as well.
+        """
+        tags: List[str] = []
+        for layer in self.parent_levels:
+            tags.extend(layer.tags or [])
+
+        tags_clean: List[str] = []
+        for tag in tags:
+            if tag not in tags_clean:
+                tags_clean.append(tag)
+        return tags_clean
 
 
-class BaseInterceptorMixin(BaseHandlerMixin):
-    def get_interceptors(self) -> List["Interceptor"]:
+class BaseInterceptorMixin(BaseHandlerMixin):  # pragma: no cover
+    def get_interceptors(self) -> List["AsyncCallable"]:
         """
         Returns all the interceptors in the handler scope from the ownsership layers.
         """
         if self._interceptors is Void:
-            self._interceptors = []
+            self._interceptors: Union[List["Interceptor"], "VoidType"] = []
             for layer in self.parent_levels:
                 self._interceptors.extend(layer.interceptors or [])
             self._interceptors = cast(
                 "List[Interceptor]",
                 [AsyncCallable(interceptors) for interceptors in self._interceptors],
             )
-        return cast("List[Interceptor]", self._interceptors)
+        return cast("List[AsyncCallable]", self._interceptors)
 
     async def intercept(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
         """
@@ -614,5 +631,5 @@ class BaseInterceptorMixin(BaseHandlerMixin):
         of the handlers.
         """
         for interceptor in self.get_interceptors():
-            interceptor = await interceptor()
-            await interceptor.intercept(scope, receive, send)
+            awaitable: "EsmeraldInterceptor" = await interceptor()
+            await awaitable.intercept(scope, receive, send)
