@@ -1,12 +1,11 @@
-import asyncio
-import functools
 import sys
 import typing
 from inspect import isclass
-from typing import Any, Awaitable, Callable, TypeVar, Union
+from typing import Any, Union
 
 import slugify
-from typing_extensions import ParamSpec, TypeGuard, get_args, get_origin
+from starlette._utils import is_async_callable as is_async_callable
+from typing_extensions import get_args, get_origin
 
 if sys.version_info >= (3, 10):
     from types import UnionType
@@ -15,16 +14,6 @@ if sys.version_info >= (3, 10):
 else:  # pragma: no cover
     UNION_TYPES = {Union}
 
-P = ParamSpec("P")
-T = TypeVar("T")
-
-
-def is_async_callable(value: Callable[P, T]) -> TypeGuard[Callable[P, Awaitable[T]]]:
-    while isinstance(value, functools.partial):
-        value = value.func  # type: ignore[unreachable]
-
-    return asyncio.iscoroutinefunction(value) or asyncio.iscoroutinefunction(value.__call__)  # ty
-
 
 def is_class_and_subclass(value: typing.Any, _type: typing.Any) -> bool:
     original = get_origin(value)
@@ -40,7 +29,7 @@ def is_class_and_subclass(value: typing.Any, _type: typing.Any) -> bool:
 
 
 def clean_string(value: str) -> str:
-    return slugify.slugify(value, separator="_")
+    return slugify.slugify(value, separator="_")  # type: ignore
 
 
 def is_optional_union(annotation: Any) -> bool:
